using System;
using System.Security.Cryptography;
using System.Text;
using System.IO;

namespace Example
{
    /// <summary>
    /// Write then verify an encrypted secret on a SmartDongle.
    ///
    /// MicroWorks, Inc.
    /// 2808 North Cole Road
    /// Boise, ID 83704
    ///
    /// http://www.smartdongle.com
    ///
    /// </summary>
    class Example3a
    {
        [STAThread]

        static void Main(string[] args)
        {
            try
            {
                UInt64 P1, P2;
                SmartDongle.SmartDnglError error;
                System.Text.ASCIIEncoding  encoding = new System.Text.ASCIIEncoding();

                /*
                 *  Padding is zeroes and CipherMode has been set to
                 *  Electronic Codebook (ECB) to match the algorithm in the C examples.
                 *
                 *  The AES key and user secret below are for example only, the user
                 *  should create a new key and secret for their application.
                 *
                 *  A System.Security.Cryptography.CryptographicException will
                 *  occur if illegal AES key length or AES key size are given below.
                */
                const string aes_key = "7obdgEn2'TH>]SE|";
                //const string aes_key = "AA09030905000697";

                const int aes_KeySize = 128;
                const int aes_BlockSize = 128;
                const string your_secret =  "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc";

                Rijndael RijndaelAlg = Rijndael.Create();
                RijndaelAlg.KeySize = aes_KeySize;
                RijndaelAlg.BlockSize = aes_BlockSize;
                RijndaelAlg.Mode = CipherMode.ECB;
                RijndaelAlg.Padding = PaddingMode.Zeros;

                /* Initial vector (rgbIV) is the same as the key as in the C example code */
                byte[] rgbKey = encoding.GetBytes(aes_key);
                byte[] rgbIV = rgbKey;

                ICryptoTransform encrypt = RijndaelAlg.CreateEncryptor(rgbKey, rgbIV);
                ICryptoTransform decrypt = RijndaelAlg.CreateDecryptor(rgbKey, rgbIV);

                /* Convert secret into byte array, then encrypt */
                byte[] plainText = Encoding.UTF8.GetBytes(your_secret);
                byte[] cipherText = encrypt.TransformFinalBlock(plainText, 0, plainText.Length);

/*
                string inFilePath = "C:\\Users\\Brandon\\Desktop\\test\\c-sharp.net.exe";
                string outFilePath = "C:\\Users\\Brandon\\Desktop\\test\\c-sharp_enc.net.exe";
                // read the bytes from the application EXE file
                FileStream fs = new FileStream(inFilePath, FileMode.Open);
                BinaryReader br = new BinaryReader(fs);
                byte[] bin = br.ReadBytes(Convert.ToInt32(fs.Length));
                fs.Close();
                br.Close();
                byte[] encryptedExe = encrypt.TransformFinalBlock(plainText, 0, plainText.Length);
                FileStream efs = new FileStream(outFilePath, FileMode.Create);
                efs.Write(encryptedExe, 0, encryptedExe.Length);
 */

                /* Your SmartDongle keys go here */
                P1 = 0xec6cc589aefd1e75;
                P2 = 0xfcec0a6a82747b3f;

                /* Write the encrypted secret to the begining of SmartDongle user memory.
                *
                *  Note: The number of writes is limited, SmartDongle uses a Catalyst
                *  CAT25C256 SPI eprom, which has a life expectancy of approximately
                *  1 Million write cycles.  Reads are unlimited.
                */
                UInt16 address = 0;
                error = SmartDongle.Write(
                    P1, P2, address, cipherText, cipherText.Length);
                if (error != SmartDongle.SmartDnglError.UskOk) {

                    System.Console.WriteLine(SmartDongle.GetErrorString(error));

                    // handle error
                    return;
                }

                /*
                 *  Do a data integrity check.
                 *  Read the encrypted secret from the SmartDongle,
                 *  decrypt and compare with original secret.
                 */
                byte[] input = new byte[cipherText.Length];
                error = SmartDongle.Read(P1, P2, address, input, cipherText.Length);
                if (error != SmartDongle.SmartDnglError.UskOk) {

                    System.Console.WriteLine(SmartDongle.GetErrorString(error));

                    // handle error
                    return;
                }

                /* Decrypt */
                byte[] result = decrypt.TransformFinalBlock(input, 0, cipherText.Length);

                /* Compare to original secret, using orignal secret length */
                int i;
                for (i = 0; (i < plainText.Length) && (result[i] == plainText[i]); i++);
                if (i == plainText.Length)
                {
                    System.Console.WriteLine("Success: Wrote Secret to SmartDongle\n");
                }
                else
                {
                    System.Console.WriteLine("Failed: Writing Secret to SmartDongle\n");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: {0}", e.Message);
            }
            System.Console.WriteLine("Press <Enter> to Exit");
            System.Console.ReadLine();
        }
    }
}

